@@ -13,6 +13,7 @@ import io
 import xml.etree.ElementTree as ET
 import logging.handlers
 from common.tc_logging import logger
+import traceback
 
 
 def nowHex():
@@ -74,6 +75,28 @@ def pool(sequence, condition=bool):
             continue
 
 
+def next_available_sip(sip_pool):
+    """Find the next available sip endpoint from a pool of endpoints"""
+    busy = True
+    a = None
+    while busy:
+        a = next(sip_pool)
+        busy = a.busy
+    if type(a).__name__ == "SipEndpointView":
+        a.busy = True
+        a.update_text()
+        a.update_arrow()
+        a.colour("green")
+    return a
+
+
+def make_available_sip(*sip_endpoints):
+    for sip_endpoint in sip_endpoints:
+        sip_endpoint.busy = False
+        if type(sip_endpoint).__name__ == "SipEndpointView":
+            sip_endpoint.colour("yellow")
+
+
 def serverThread(target, *args, **kwargs):
     """ Start a thread """
     ex = ThreadPoolExecutor()
@@ -92,7 +115,7 @@ def wait_for_sip_data(sockfile):
         if not line.strip():
             break
         header, value = [x.strip() for x in line.split(b":", 1)]
-        if header == b"Content-Length":
+        if header == b"Content-Length" or header == b"content-length":
             content_length = int(value)
 
     if content_length > 0:
@@ -322,21 +345,24 @@ class Load(object):
         handler = logging.handlers.RotatingFileHandler("Statistics.txt", mode="w", maxBytes=20000000, backupCount=5)
         st_logger.addHandler(handler)
         self.log = st_logger
-        self.calls = {"Started": 0, "Finished": 0}
+        self.calls = {"Started": 0, "Finished": 0, "Passed": 0}
+        self.loop = LoadThread(target=self._start)
         self.statistics()
 
     def start(self):
+        self.loop.start()
+
+    def _start(self):
         """
         Every :interval seconds, start :quantity flows
         """
-        t = time()
-        for i in range(self.quantity):
-            self.run_next_flow()
-        if self.stopCondition or not (self.duration < 0 or time() - self.startTime < self.duration):
-            self.stop()
-        else:
-            sleep(max((0, self.interval-time()+t)))
-            self.start()
+        while not (self.stopCondition or not (self.duration < 0 or time() - self.startTime < self.duration)):
+            t = time()
+            for i in range(self.quantity):
+                self.run_next_flow()
+                sleep(max((0, self.interval-time()+t)))
+
+        self.stop()
 
     def stop(self):
         """
@@ -351,16 +377,27 @@ class Load(object):
         self.active.append(c)
         self.calls["Started"] += 1
 
-    def monitor(self):
+    def monitor(self, max_tracked_errors=30):
         while self.active or not self.stopCondition:
+            sleep(0.1)
             for inst in (ins for ins in self.active if not ins.is_alive()):
                 try:
                     inst.join()
-                except:
-                    pass
+                except Exception as e:
+                    key = str(e)
+                    if ": Got" in key:
+                        key = key.split('"')[1]
+                    key = "Failed: " + key
+                    if key in self.calls:
+                        self.calls[key] += 1
+                    elif len(self.calls) < max_tracked_errors:
+                        self.calls[key] = 1
+                else:
+                    self.calls["Passed"] += 1
                 finally:
                     self.active.remove(inst)
                     self.calls["Finished"] += 1
+        self.loop.join()
 
     def statistics(self):
         self.log.info("{}:{}".format(time(), self.calls))
@@ -379,6 +416,7 @@ class LoadThread(Thread):
             super().run()
         except Exception as e:
             logger.debug("Exception in Thread")
+            logger.debug(traceback.format_exc())
             self.exc = e
             raise
 
